@@ -69,6 +69,7 @@ has 'children' => (
     traits => ['Array'],
     handles => {
         add_child => 'push',
+        has_children => 'count',
     },
 );
 
@@ -88,6 +89,11 @@ has 'value' => (
     isa => 'Maybe[Str]',
 );
 
+has 'last_seen_date' => (
+    is => 'rw',
+    isa => 'Maybe[DateTime]',
+);
+
 sub add_property {
     my $self = shift;
 
